﻿'*
'*  4dSystems製 uOLED-96-G1 のSDカード内のイメージ表示コマンドに対応できる様に
'*　FONTX形式のフォントデータを1セクタ毎に展開したイメージファイルを作成する
'*  ここでいうセクタは論理セクタ番号なのでマイコン側で物理セクタ番号に変換する
'*
'*  ASCIIコード 0x00 ～ 0xff
'*  S-JISコード 0x8140 ～ 0x9fff / 0xe040 ～ 0xeaa4
'*
'*  http://www.softclub.jp/~zoro/ のフォント「X11 bdf形式 (k12x10bdf.tar.gz, 155Kbyte) 」で動作確認している
'*
'*  セクタ０
'*  0x0000( 16 ) "OLED_FONTS"            識別文字列 + 0x00 ...
'*  0x0010( 16 ) "2008/10/01"            作成年月日 + 0x00 ...
'*  0x0020(  1 ) フォントブロック数      最大32(3固定...マイコンの負荷を軽減する為)
'*  0x0021(  1 ) フォント色モード   8 or 16
'*
'*  セクタ１（フォントブロック数分のブロックデータが続く(3ブロック固定）
'*  0x0000( 6) : "BLOCKn"   ブロック識別子
'*  0x0006( 4) : フォントデータセクタ番号
'*  0x000A( 2) : 開始文字コード
'*  0x000C( 2) : 終了文字コード
'*  0x000E( 1) : フォントXサイズ
'*  0x000F( 1) : フォントYサイズ
'*
'*  セクタ２～　フォントデータ
'* 

Public Class fmFontMake

    Private defaultMessage As String = "ASCII 0x0000 ～ 0x00ff" & vbCr & "S-JIS 0x8140 ～ 0x9fff / 0xe040 ～ 0xeaa4"

    '*************************************************************************
    '*  イベント
    '*                          フォームロード
    '*************************************************************************
    Private Sub fmFontMake_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        '*  実行フォルダ内にK6X10.FNT / K12X10.FNTファイルがみつかった場合は
        '*  そのパスをテキストボックスに設定
        Dim p As String = Application.StartupPath
        If (p.Substring(p.Length - 1) <> "\") Then
            p += "\"
        End If

        If (System.IO.File.Exists(p & "k6x10.fnt") = True) Then
            Me.TextBox_ASCII.Text = p & "k6x10.fnt"
        End If

        If (System.IO.File.Exists(p & "k12x10.fnt") = True) Then
            Me.TextBox_KANJI.Text = p & "k12x10.fnt"
        End If

        '*  プレビュー用ピクチャボックスグリッド作成
        functions.makePiclist(Me.Panel1, 30, 14, 32, 32, 0, 0)

        '*  デフォルトメッセージ設定
        Me.LabelMessage.Text = Me.defaultMessage

        Me.PictureBox2.AllowDrop = True

        fmIconMake.Hide()
        fmOgg.Hide()

    End Sub

    '*************************************************************************
    '*  イベント
    '*                          フォームクローズ
    '*************************************************************************
    Private Sub fmFontMake_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        End

    End Sub

    '*************************************************************************
    '*  イベント
    '*              メニュー「アイコンファイル作成」クリック
    '*************************************************************************
    Private Sub IconToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles IconToolStripMenuItem.Click
        fmIconMake.Top = Me.Top
        fmIconMake.Left = Me.Left
        Me.Hide()
        fmIconMake.Show()
    End Sub

    '*************************************************************************
    '*  イベント
    '*              メニュー「OGGプラグインファイル作成」クリック
    '*************************************************************************
    Private Sub OggToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OggToolStripMenuItem.Click
        fmOgg.Top = Me.Top
        fmOgg.Left = Me.Left
        Me.Hide()
        fmOgg.Show()
    End Sub

    '*************************************************************************
    '*  イベント
    '*                      フォント色変更用ボタンクリック
    '*************************************************************************
    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click

        fmColor.ShowDialog()
        If (fmColor.DialogResult = Windows.Forms.DialogResult.OK) Then
            Me.PictureBox1.BackColor = fmColor.GetValue()
        End If
        fmColor.Dispose()

    End Sub

    '*************************************************************************
    '*  イベント
    '*                        背景色変更用ボタンクリック
    '*************************************************************************
    Private Sub Button5_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button5.Click

        If (Not Me.PictureBox2.Image Is Nothing) Then
            If (MessageBox.Show("現在背景にイメージファイルが読み込まれています" & vbCr & vbLf & "背景色を変更するとイメージは消去されますが宜しいですか？", "", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = Windows.Forms.DialogResult.No) Then
                Return
            End If
        End If

        fmColor.ShowDialog()
        If (fmColor.DialogResult = Windows.Forms.DialogResult.OK) Then
            If (Not Me.PictureBox2.Image Is Nothing) Then
                Me.PictureBox2.Image.Dispose()
                Me.PictureBox2.Image = Nothing
            End If
            Me.PictureBox2.BackColor = fmColor.GetValue()
        End If
        fmColor.Dispose()

    End Sub

    '*************************************************************************
    '*  イベント
    '*                 ASCIIフォントファイル選択ボタンクリック
    '*************************************************************************
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click

        Using d As New OpenFileDialog
            With d
                .Title = "FONTX ASCII フォントファイル選択"
                .Filter = "FONTXファイル(*.fnt)|*.fnt|すべてのファイル(*.*)|*.*"
                .FilterIndex = 0
                If (Me.TextBox_ASCII.Text.Length <= 0) Then
                    .FileName = "k6x10.fnt"
                Else
                    .FileName = Me.TextBox_ASCII.Text
                End If

                If (.ShowDialog() = DialogResult.OK) Then
                    Me.TextBox_ASCII.Text = .FileName
                End If
            End With
        End Using

    End Sub

    '*************************************************************************
    '*  イベント
    '*                  漢字フォントファイル選択ボタンクリック
    '*************************************************************************
    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click

        Using d As New OpenFileDialog
            With d
                .Title = "FONTX 漢字フォントファイル選択"
                .Filter = "FONTXファイル(*.fnt)|*.fnt|すべてのファイル(*.*)|*.*"
                .FilterIndex = 0
                If (Me.TextBox_KANJI.Text.Length <= 0) Then
                    .FileName = "k12x10.fnt"
                Else
                    .FileName = Me.TextBox_KANJI.Text
                End If

                If (.ShowDialog() = DialogResult.OK) Then
                    Me.TextBox_KANJI.Text = .FileName
                End If
            End With
        End Using

    End Sub

    '*************************************************************************
    '*  イベント
    '*                        フォント作成ボタンクリック
    '*************************************************************************
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click

        '*  コントロール無効
        Me.Buttons_Enabled(False)

        '* ASCIIフォント読み込み
        Dim fxAscii As fontx = Me.loadFontx(Me.TextBox_ASCII.Text)
        If (fxAscii Is Nothing) Then
            Me.Buttons_Enabled(True)
            Return
        End If

        If (fxAscii.type <> 0) Then
            MessageBox.Show("ASCIIフォントではないフォントが指定されています", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Buttons_Enabled(True)
            Return
        End If

        '* 漢字フォント読み込み
        Dim fxKanji As fontx = Me.loadFontx(Me.TextBox_KANJI.Text)
        If (fxAscii Is Nothing) Then
            Me.Buttons_Enabled(True)
            Return
        End If

        If (fxKanji.type <> 1) Then
            MessageBox.Show("漢字フォントではないフォントが指定されています", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Me.Buttons_Enabled(True)
            Return
        End If

        '*  背景イメージサイズチェック
        If (Not Me.PictureBox2.Image Is Nothing) Then
            Dim bg As Bitmap = CType(Me.PictureBox2.Image, Bitmap)
            If ((bg.Width < fxAscii.xsize) OrElse (bg.Width < fxKanji.xsize)) Then
                MessageBox.Show("背景に設定されているイメージがフォントサイズを下回ります(横)", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Me.Buttons_Enabled(True)
                Return
            End If
            If ((bg.Height < fxAscii.ysize) OrElse (bg.Height < fxKanji.ysize)) Then
                MessageBox.Show("背景に設定されているイメージがフォントサイズを下回ります(縦)", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Me.Buttons_Enabled(True)
                Return
            End If
        End If

        '*  メッセージ表示
        Me.LabelMessage.Text = "フォントデータサイズ計算中"
        Application.DoEvents()

        '*  フォントブロックデータ定義
        Dim startCode() As UShort = {&H0, &H8140, &HE040}
        Dim endCode() As UShort = {&HFF, &H9FFF, &HEAA4}
        Dim fx() As fontx = {fxAscii, fxKanji, fxKanji}

        '*  プログレスバーパラメータ作成
        Me.ProgressBar1.Value = 0
        Me.ProgressBar1.Minimum = 0
        Me.ProgressBar1.Maximum = 0
        For i As Integer = 0 To startCode.Length - 1
            Me.ProgressBar1.Maximum += CType((endCode(i) - startCode(i)), Integer)
            Me.ProgressBar1.Maximum += 1
        Next i

        '*  フォントファイルデータ領域確保（1ブロック512Byte)
        Dim secCount As UInt32 = 2
        For i As Integer = 0 To startCode.Length - 1
            Me.setFontData(secCount, Nothing, fx(i), startCode(i), endCode(i), False)
        Next i
        Dim fontDatas((secCount * 512) - 1) As Byte
        Array.Clear(fontDatas, 0, fontDatas.Length)

        '*  メッセージ更新
        Me.LabelMessage.Text = "フォントデータ作成中"
        Application.DoEvents()

        '*  ファイルヘッダ設定
        strcpy(fontDatas, &H0, "OLED_FONTS")
        strcpy(fontDatas, &H10, Now.ToString("yyyy/MM/dd"))

        fontDatas(&H20) = CType(startCode.Length, Byte)       '* フォントブロック数
        If (Me.RadioButton1.Checked = True) Then              '* カラーコードビット数
            fontDatas(&H21) = 8
        Else
            fontDatas(&H21) = 16
        End If

        '*  フォントデータ設定
        Dim secIndex As UInt32 = 2
        For i As Integer = 0 To startCode.Length - 1

            '*  ブロック識別子
            Dim pos As UInt32 = 512 + (i * 16)
            strcpy(fontDatas, pos, "BLOCK")
            fontDatas(pos + 5) = Asc(i.ToString)

            '* セクタオフセット値
            fontDatas(pos + 6) = CType((secIndex And &HFF), Byte)
            fontDatas(pos + 7) = CType(((secIndex >> 8) And &HFF), Byte)
            fontDatas(pos + 8) = CType(((secIndex >> 16) And &HFF), Byte)
            fontDatas(pos + 9) = CType(((secIndex >> 24) And &HFF), Byte)

            '* 開始コード
            fontDatas(pos + 10) = CType((startCode(i) And &HFF), Byte)
            fontDatas(pos + 11) = CType(((startCode(i) And &HFF00) >> 8), Byte)

            '* 終了コード
            fontDatas(pos + 12) = CType((endCode(i) And &HFF), Byte)
            fontDatas(pos + 13) = CType(((endCode(i) And &HFF00) >> 8), Byte)

            '* フォントサイズ
            fontDatas(pos + 14) = fx(i).xsize
            fontDatas(pos + 15) = fx(i).ysize

            '*  フォントデータ
            Me.setFontData(secIndex, fontDatas, fx(i), startCode(i), endCode(i), True)

        Next i

        '*  フォントデータをファイルに書き込む
        Using d As New SaveFileDialog
            With d
                .FileName = "font.dat"
                .Filter = "DATファイル(*.dat)|*.dat|すべてのファイル(*.*)|*.*"
                .Title = "フォント保存先のファイルを選択してください"
                .RestoreDirectory = True
                If (.ShowDialog() = DialogResult.OK) Then
                    Me.LabelMessage.Text = "フォントデータ保存中"
                    Application.DoEvents()
                    Dim bw As New System.IO.FileStream(.FileName, IO.FileMode.Create, IO.FileAccess.Write)
                    bw.Write(fontDatas, 0, fontDatas.Length)
                    bw.Close()
                    bw.Dispose()
                    MessageBox.Show("フォントファイルを作成しました", "", MessageBoxButtons.OK, MessageBoxIcon.Information)
                End If
            End With
        End Using

        '*  メッセージリセット
        Me.LabelMessage.Text = Me.defaultMessage

        '*  コントロール有効
        Me.Buttons_Enabled(True)

    End Sub

    '*************************************************************************
    '*  イベント
    '*          ピクチャボックスグリッドへドロップができるかチェック
    '*************************************************************************
    Private Sub PictureBox2_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles PictureBox2.DragEnter

        e.Effect = DragDropEffects.None

        Dim bmp As Bitmap = Nothing
        If (e.Data.GetDataPresent(DataFormats.FileDrop)) Then
            Dim fnames() As String = CType(e.Data.GetData(DataFormats.FileDrop), String())
            If (fnames.Length = 1) Then
                bmp = New Bitmap(fnames(0))
                If (Not bmp Is Nothing) Then
                    If ((bmp.Width > 0) AndAlso (bmp.Width < 32) OrElse (bmp.Height > 0) AndAlso (bmp.Height < 32)) Then
                        e.Effect = DragDropEffects.Copy
                    End If
                End If
            End If
        End If

    End Sub

    '*************************************************************************
    '*  イベント
    '*                   ピクチャボックスグリッドへドロップ
    '*************************************************************************
    Private Sub PictureBox2_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles PictureBox2.DragDrop

        Dim bmp As Bitmap = Nothing
        Dim img As Bitmap = Nothing

        If (e.Data.GetDataPresent(DataFormats.FileDrop)) Then
            Dim fnames() As String = CType(e.Data.GetData(DataFormats.FileDrop), String())
            If (fnames.Length = 1) Then
                Try
                    img = New Bitmap(fnames(0))
                    bmp = New Bitmap(img.Width, img.Height, Imaging.PixelFormat.Format24bppRgb)
                    Using g As Graphics = Graphics.FromImage(bmp)
                        g.FillRectangle(Brushes.White, New RectangleF(0, 0, bmp.Width, bmp.Height))
                    End Using
                    For y As Integer = 0 To bmp.Height - 1
                        For x As Integer = 0 To bmp.Width - 1
                            bmp.SetPixel(x, y, img.GetPixel(x, y))
                        Next x
                    Next y
                    Me.PictureBox2.Image = bmp.Clone
                Catch ex As Exception
                End Try

            End If
        End If

    End Sub

    '*************************************************************************
    '*  メソッド
    '*                  各ボタンコントロールの有効／無効設定
    '*  引数：Boolean Enabled True:有効  False:無効
    '*  戻値：void
    '*************************************************************************
    Private Sub Buttons_Enabled(ByVal Enabled As Boolean)

        Me.Button1.Enabled = Enabled
        Me.Button2.Enabled = Enabled
        Me.Button3.Enabled = Enabled
        Me.Button4.Enabled = Enabled
        Me.Button5.Enabled = Enabled
        Me.RadioButton1.Enabled = Enabled
        Me.RadioButton2.Enabled = Enabled
        Me.FileToolStripMenuItem.Enabled = Enabled

    End Sub

    '*************************************************************************
    '*  メソッド
    '*                  フォントファイル領域にフォントデータを設定
    '*  引数：ByRef UInt32 secIndex    セクタインデックス(作成したセクタ数分が自動加算される）
    '*        Byte         fontDatas() フォントファイルデータ領域
    '*        fontx        fx          フォントデータ
    '*        UShort       startCode   開始文字コード
    '*        UShort       endCode     終了文字コード
    '*        Boolean      mode        動作モード(False:計算オンリー / True:実データ作成)
    '*  戻値：void
    '*
    '*  例）フォントの横ドット数が4ドットの場合は以下の様なデータが作成される
    '*  X : 0 ～ 0
    '*  X : 0 ～ 1
    '*  X : 0 ～ 2
    '*  X : 0 ～ 3  フォントの全形
    '*
    '*************************************************************************
    Private Sub setFontData(ByRef secIndex As UInt32, ByRef fontDatas As Byte(), ByVal fx As fontx, ByVal startCode As UShort, ByVal endCode As UShort, ByVal mode As Boolean)

        '* ドット辺りのバイト数を取得
        Dim ByteOfColor As Integer = 0
        If (Me.RadioButton1.Checked = True) Then
            ByteOfColor = 1
        Else
            ByteOfColor = 2
        End If

        '*  メインループ
        For code As Integer = CType(startCode, Integer) To CType(endCode, Integer)

            '*  ビットマップ作成
            Dim bmp As Bitmap = fx.getFontImage(CType(code, UShort), Me.PictureBox1.BackColor, Me.PictureBox2)

            '*  フォント展開
            For i As Integer = 0 To fx.xsize - 1
                Dim pos As UInt32 = secIndex * 512

                For y As Integer = 0 To fx.ysize - 1
                    For x As Integer = 0 To i
                        If (ByteOfColor = 1) Then
                            If (mode = True) Then
                                fontDatas(pos) = ColorToRGB8(bmp.GetPixel(x, y))
                            End If
                            pos += 1
                        Else
                            If (mode = True) Then
                                Dim c As UShort = ColorToRGB16(bmp.GetPixel(x, y))
                                fontDatas(pos + 0) = CType(((c And &HFF00) >> 8), Byte)
                                fontDatas(pos + 1) = CType((c And &HFF), Byte)
                            End If
                            pos += 2
                        End If
                    Next x
                Next y

                secIndex += 1
            Next i

            '*  プレビュー表示
            If (mode = True) Then
                CType(Me.Panel1.Controls(CType(Me.ProgressBar1.Value, Integer) Mod Me.Panel1.Controls.Count), PictureBox).Image = bmp.Clone
                If ((Me.ProgressBar1.Value Mod 25) = 0) Then
                    Application.DoEvents()
                End If
                Me.ProgressBar1.Value += 1
            End If

        Next code

    End Sub

    '*************************************************************************
    '*  メソッド
    '*                   指定のFONTX形式ファイルを読み込む
    '*  引数：String fname FONTX形式ファイル名
    '*  戻値：fontx        読み込まれたフォント情報
    '*************************************************************************
    Private Function loadFontx(ByVal fname As String) As fontx

        '*  FONTX読み込み
        Dim fx As fontx = Nothing
        Try
            fx = New fontx(fname)
        Catch ex As Exception
            MessageBox.Show(System.IO.Path.GetFileNameWithoutExtension(fname) & " の読み込みに失敗しました", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return (Nothing)
        End Try

        '*  ドット辺りのバイト数計算
        Dim ByteOfDot As Integer = 1
        If (Me.RadioButton2.Checked = True) Then
            ByteOfDot = 2
        End If

        '*  データチェック
        If (fx.xsize * fx.ysize * ByteOfDot) > 512 Then
            MessageBox.Show("1ブロック(512Byte)に収まらないフォントの為、nPMPでは使用できません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return (Nothing)
        End If

        Return (fx)

    End Function

End Class