﻿'*****************************************************************************
'*  クラス
'*                      FONTX形式ファイル構造解析クラス
'*****************************************************************************
Public Class fontx

    '*  ローカル変数宣言
    Private _FontName As String             '* フォント名
    Private _Xdot As Integer = 0            '* フォント横ドット数
    Private _Ydot As Integer = 0            '* フォント縦ドット数
    Private _CodeType As Integer = -1       '* フォントタイプ 0:ASCII 1:漢字

    Private _Base As Long = 0               '* フォントデータの始まり位置
    Private _chByte As Integer = 0          '* 1文字辺りのバイト数
    Private _lnByte As Integer = 0          '* 1ライン辺りのバイト数

    Private _Start(256) As UShort           '* エントリー
    Private _End(256) As UShort
    Private _Ent As Integer = 0

    Private _fx() As Byte                   '* FONTXデータ

    '*************************************************************************
    '*  コンストラクタ
    '*  引数：String fname FONTX形式のファイル名
    '*************************************************************************
    Sub New(ByVal fname As String)

        Dim fs As System.IO.FileStream = Nothing
        Dim fpos As Long = 0

        '*
        '*  FONTXファイルを読み込む
        '*
        Try
            fs = New System.IO.FileStream(fname, IO.FileMode.Open, IO.FileAccess.Read)
            If (fs.Length < 17) Then
                MessageBox.Show("ヘッダサイズ分のファイルサイズがありません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Error 1
                Return
            End If

            ReDim Me._fx(fs.Length)
            fs.Read(Me._fx, 0, fs.Length)
            fs.Close()
            fs.Dispose()
            fs = Nothing

        Catch ex As Exception
            MessageBox.Show(fname & "を読み込めません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Error 1
            Return
        End Try

        '*
        '*  ヘッダ解析
        '*
        Dim fontx2 As String = System.Text.Encoding.GetEncoding(932).GetString(Me._fx, 0, 6)
        If (fontx2 <> "FONTX2") Then
            MessageBox.Show("ファイル識別子「FONTX2」を確認できません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Error 1
            Return
        End If

        Me._FontName = System.Text.Encoding.GetEncoding(932).GetString(Me._fx, 6, 8).Trim
        Me._Xdot = CType(Me._fx(14), Integer)
        If ((Me._Xdot < 1) OrElse (Me._Xdot > 28)) Then
            MessageBox.Show("フォントＸサイズ " & Me._Xdot.ToString & "はサポートしません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Error 1
            Return
        End If

        Me._Ydot = CType(Me._fx(15), Integer)
        If ((Me._Ydot < 1) OrElse (Me._Ydot > 28)) Then
            MessageBox.Show("フォントＹサイズ " & Me._Xdot.ToString & "はサポートしません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Error 1
            Return
        End If

        Me._CodeType = CType(Me._fx(16), Integer)
        If ((Me._CodeType <> 0) AndAlso (Me._CodeType <> 1)) Then
            MessageBox.Show("コードタイプ0x" & Me._CodeType.ToString("02x") & "はサポートしません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Error 1
            Return
        End If

        '*
        '*  フォントエントリー取得
        '*
        If (Me._CodeType = 0) Then
            Me._Ent = 1
            Me._Start(0) = 0
            Me._End(0) = 255
            Me._Base = 17
        Else
            Me._Ent = CType(Me._fx(17), Integer)
            If ((Me._Ent < 1) OrElse (Me._Ent > 255)) Then
                MessageBox.Show("エントリー数 " & Me._Ent.ToString & "はサポートしません", "", MessageBoxButtons.OK, MessageBoxIcon.Error)
                Error 1
                Return
            End If

            Me._Base = 18
            For i As Integer = 0 To Me._Ent - 1
                Me._Start(i) = (CType(Me._fx(Me._Base + 1), UShort) << 8) Or CType(Me._fx(Me._Base + 0), UShort)
                Me._End(i) = (CType(Me._fx(Me._Base + 3), UShort) << 8) Or CType(Me._fx(Me._Base + 2), UShort)
                Me._Base += 4
            Next i

        End If

        '*
        '*  パラメタ作成
        '*
        Me._lnByte = (Me._Xdot + 7) \ 8
        Me._chByte = Me._Ydot * Me._lnByte

    End Sub

    '*************************************************************************
    '*  プロパティ
    '*                        １文字辺りのバイト数
    '*  引数：void
    '*  戻値：Integer
    '*************************************************************************
    Public ReadOnly Property ByteOfFont() As Integer
        Get
            Return (Me._chByte)
        End Get
    End Property

    '*************************************************************************
    '*  プロパティ
    '*                               ライン辺りのバイト数
    '*  引数：void
    '*  戻値：Integer
    '*************************************************************************
    Public ReadOnly Property ByteOfLine() As Integer
        Get
            Return (Me._lnByte)
        End Get
    End Property

    '*************************************************************************
    '*  プロパティ
    '*                               横ドット数
    '*  引数：void
    '*  戻値：Integer
    '*************************************************************************
    Public ReadOnly Property xsize() As Integer
        Get
            Return (Me._Xdot)
        End Get
    End Property

    '*************************************************************************
    '*  プロパティ
    '*                               縦ドット数
    '*  引数：void
    '*  戻値：Integer
    '*************************************************************************
    Public ReadOnly Property ysize() As Integer
        Get
            Return (Me._Ydot)
        End Get
    End Property

    '*************************************************************************
    '*  プロパティ
    '*                               フォントタイプ
    '*  引数：void
    '*  戻値：Integer   0:半角(ASCII)フォント 1:全角(S-JIS)フォント
    '*************************************************************************
    Public ReadOnly Property type() As Integer
        Get
            Return (Me._CodeType)
        End Get
    End Property

    '*************************************************************************
    '*  プロパティ
    '*                               フォント名
    '*  引数：void
    '*  戻値：String
    '*************************************************************************
    Public ReadOnly Property name() As String
        Get
            Return (Me._FontName)
        End Get
    End Property

    '*************************************************************************
    '*  メソッド
    '*              指定文字コードのフォントデータの先頭アドレスを取得
    '*  引数：UShort Code 文字コード( 0x0000 ～ 0xffff )
    '*  戻値：long        先頭インデックス(0:データ無し)
    '*************************************************************************
    Public Function getFontIndex(ByVal Code As UShort) As Long

        Dim pos As Long = Me._Base
        For i As Integer = 0 To Me._Ent - 1
            Dim s As UShort = Me._Start(i)
            Dim e As UShort = Me._End(i)
            If ((Code >= s) AndAlso (Code <= e)) Then
                Return (pos + (CType((Code - s), Long) * Me._chByte))
            End If
            pos += ((CType((e - s), Long) + 1) * Me._chByte)
        Next i

        Return (0)

    End Function

    '*************************************************************************
    '*  メソッド
    '*                    指定文字コードの生データを取得
    '*  引数：UShort Code 文字コード( 0x0000 ～ 0xffff )
    '*        Byte data() 格納領域
    '*  戻値：void
    '*************************************************************************
    Public Sub getFontData(ByVal Code As UShort, ByRef data() As Byte)

        '*  フォントエントリから指定のコードを検索
        Dim pos As Long = getFontIndex(Code)

        '*  フォントデータを複写
        For i As Integer = 0 To Me._chByte - 1
            data(i) = Me._fx(pos + i)
        Next i

    End Sub

    '*************************************************************************
    '*  メソッド
    '*                    指定文字コードのイメージを取得
    '*  引数：UShort     Code 文字コード( 0x0000 ～ 0xffff )
    '*        Color      fc   文字色
    '*        PictureBox bc   背景色
    '*  戻値：Bitmap          画像データ
    '*************************************************************************
    Public Function getFontImage(ByVal Code As UShort, ByVal fc As Color, ByVal bc As PictureBox) As Bitmap

        '*  フォントエントリから指定のコードを検索
        Dim pos As Long = getFontIndex(Code)

        '*  背景で塗りつぶしたビットマップを作成
        Dim bmp As New Bitmap(Me._Xdot, Me._Ydot, Imaging.PixelFormat.Format24bppRgb)

        If (Not bc.Image Is Nothing) Then
            For y As Integer = 0 To Me._Ydot - 1
                For x As Integer = 0 To Me._Xdot - 1
                    bmp.SetPixel(x, y, CType(bc.Image, Bitmap).GetPixel(x, y))
                Next x
            Next y
        Else
            Using b As New SolidBrush(bc.BackColor)
                Using g As Graphics = Graphics.FromImage(bmp)
                    g.FillRectangle(b, New RectangleF(0, 0, Me._Xdot, Me._Ydot))
                End Using
            End Using
        End If

        '*  FONTXのビットマップｗ展開
        If (pos <> 0) Then
            For y As Integer = 0 To Me._Ydot - 1
                For x As Integer = 0 To Me._Xdot - 1
                    If Me._fx(pos + (x \ 8)) And (2 ^ (7 - (x Mod 8))) Then
                        bmp.SetPixel(x, y, fc)
                    End If
                Next x
                pos += Me._lnByte
            Next y
        End If

        Return (bmp)

    End Function

End Class
