﻿Module functions

    Private Function SCALE_CONVERT(ByVal v As UInt32, ByVal rg1 As UInt32, ByVal rg2 As UInt32) As UInt32
        Return (((((v * rg2) * 10) + 5) \ rg1) \ 10)
    End Function

    '*************************************************************************
    '*  関数
    '*           OLEDの色コード(8bit)をColorオブジェクトに変換
    '*  引数：Byte  c8 OLED8ビット色コード
    '*  戻値：Color Colorオブジェクト
    '*************************************************************************
    Public Function RBG8ToColor(ByVal c8 As Byte) As Color

        Dim r As Byte = SCALE_CONVERT(CType(((c8 And &HE0) >> 5), UInt32), 8, 256)
        Dim g As Byte = SCALE_CONVERT(CType(((c8 And &H1C) >> 2), UInt32), 8, 256)
        Dim b As Byte = SCALE_CONVERT(CType((c8 And &H3), UInt32), 4, 256)
        Return (ColorTranslator.FromWin32(RGB(r, g, b)))

    End Function

    '*************************************************************************
    '*  関数
    '*           OLEDの色コード(16bit)をColorオブジェクトに変換
    '*  引数：UShort c16 OLED16ビット色コード
    '*  戻値：Color  Colorオブジェクト
    '*************************************************************************
    Public Function RBG16ToColor(ByVal c16 As UShort) As Color

        Dim r As Byte = SCALE_CONVERT(CType(((c16 And &HF800) >> 11), UInt32), 32, 256)
        Dim g As Byte = SCALE_CONVERT(CType(((c16 And &H7E0) >> 5), UInt32), 64, 256)
        Dim b As Byte = SCALE_CONVERT(CType((c16 And &H1F), UInt32), 32, 256)
        Return (ColorTranslator.FromWin32(RGB(r, g, b)))

    End Function

    '*************************************************************************
    '*  関数
    '*           ColorオブジェクトをOLEDの色コード(8bit)に変換
    '*  引数：Color c 色オブジェクト
    '*  戻値：Byte    OLED形式の色コード
    '*************************************************************************
    Public Function ColorToRGB8(ByVal c As Color) As Byte

        Dim r As UInt32 = SCALE_CONVERT(CType(c.R, UInt32), 256, 8)
        Dim g As UInt32 = SCALE_CONVERT(CType(c.G, UInt32), 256, 8)
        Dim b As UInt32 = SCALE_CONVERT(CType(c.B, UInt32), 256, 4)
        Return ((r * 32) + (g * 4) + b)

    End Function

    '*************************************************************************
    '*  関数
    '*           ColorオブジェクトをOLEDの色コード(16bit)に変換
    '*  引数：Color c 色オブジェクト
    '*  戻値：UShort  OLED形式の色コード
    '*************************************************************************
    Public Function ColorToRGB16(ByVal c As Color) As UShort

        Dim r As UInt32 = SCALE_CONVERT(CType(c.R, UInt32), 256, 32)
        Dim g As UInt32 = SCALE_CONVERT(CType(c.G, UInt32), 256, 64)
        Dim b As UInt32 = SCALE_CONVERT(CType(c.B, UInt32), 256, 32)
        Return ((r * 2048) + (g * 32) + b)

    End Function

    '*************************************************************************
    '*  サブプロシジャ
    '*           指定のパネル内にピクチャボックスでグリッドを作成
    '*  引数：Panel   pn    パネルコントロール
    '*        Integer xcnt  作成する横の数
    '*        Integer ycnt  作成する縦の数
    '*        Integer xsize 作成するピクチャボックスの横ドット数
    '*        Integer ysize 作成するピクチャボックスの縦ドット数
    '*        Integer gap   ピクチャボックス間のギャップドット数
    '*  戻値：void
    '*************************************************************************
    Public Sub makePiclist(ByVal pn As Panel, ByVal xcnt As Integer, ByVal ycnt As Integer, ByVal xsize As Integer, ByVal ysize As Integer, ByVal gapx As Integer, ByVal gapy As Integer)

        pn.Visible = False
        pn.Enabled = False

        pn.Controls.Clear()

        Dim Index As Integer = 0
        For y As Integer = 1 To ycnt
            Dim cy As Integer = 2 + ((y - 1) * (gapy + ysize))

            For x = 1 To xcnt
                Dim cx As Integer = 2 + ((x - 1) * (gapx + xsize))
                Dim pic As New PictureBox
                pic.Top = cy
                pic.Left = cx
                pic.Width = xsize
                pic.Height = ysize
                pic.SizeMode = PictureBoxSizeMode.CenterImage
                pic.BorderStyle = BorderStyle.Fixed3D
                pn.Controls.Add(pic)

                If (gapy > 16) Then
                    Dim lb As New Label
                    lb.Font = New Font("MS UI Gothic", 8)
                    lb.Text = "No." & Index.ToString
                    lb.Top = cy + ysize
                    lb.Left = cx
                    pn.Controls.Add(lb)
                    Index += 1
                End If

            Next x

        Next y

        pn.Enabled = True
        pn.Visible = True

    End Sub

    '*************************************************************************
    '*         指定文字列(VB.NET(UCODE))　⇒　指定の配列にセット(S-JIS)
    '*  引数：Byte buf()   格納バッファ
    '*        Integer spos 格納バッファオフセット
    '*        String str   変換する文字列
    '*  戻値：void
    '*************************************************************************
    Public Sub strcpy(ByVal buf As Byte(), ByVal spos As Integer, ByVal str As String)
        Dim d As Byte() = System.Text.Encoding.GetEncoding(932).GetBytes(str)   '* S-JIS変換
        For i As Integer = 0 To d.Length - 1
            buf(spos + i) = d(i)
        Next i
    End Sub

End Module
